library docs_analyzer.bin;

import 'dart:io';

import 'package:cli_util/cli_util.dart' as cli_util;
import 'package:docs_analyzer/docs_analyzer.dart';

main(List<String> arguments) async {
  Directory inputDir = _parseArguments(arguments);
  Directory sdkDir = cli_util.getSdkDir(arguments);

  var docsAnalysis = await analyzeDocumentation(inputDir, sdkDir);

  if (docsAnalysis.isMissingDocumentation) {
    print('Some top-level elements are missing documentation.');
    docsAnalysis.undocumentedElements.forEach((elem) => print(elem.fullyQualifiedName));
  } else {
    print('All top-level elements are documented.');
  }

  _exit(docsAnalysis.isMissingDocumentation);
}

Directory _parseArguments(List<String> args) {
  if (args.length > 1) {
    _printHelpAndExit('Wrong number of arguments');
  }

  if (args.length == 0) {
    return Directory.current;
  }

  if (args[0] == '--help' || args[0] == '-h') {
    _printHelpAndExit();
  }

  return new Directory(args[0]);
}


void _printHelpAndExit([String message]) {
  if (message != null) {
    print('\n' + message);
  }

  final String usage = [
    "",
    "docs_analyzer: Analyzes Dart projects for top-level elements missing documentation.",
    "",
    "Usage:",
    "  docs_analyzer <project_path>     Run docs_analyzer on the project located at <project_path>",
    "  docs_analyzer                    Run docs_analyzer on the current directory",
    "  docs_analyzer -h | --help        Show this screen",
    ""
  ].join('\n');

  print(usage);
  _exit(message != null);
}

_exit(bool error) {
  exit(error ? 1 : 0);
}
